/* ---------------------------------------------------------------------------
 *  Tseng Labs ET6000 fb operators
 * --------------------------------------------------------------------------
 *
 *  Copyright (C) 1997, The Linux-GGI Project
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * ---------------------------------------------------------------------------
 *
 *  $Log: et6000_fb.inc,v $
 *
 * ---------------------------------------------------------------------------
 *
 *  This code was derived from the following sources of information:
 *  [1]     IBM VGA driver
 *  [2]     Prelease VGADOC5 by Finn Thoegersen
  */

#ifndef _ET6000_FB_INC_
#define _ET6000_FB_INC_

/* Smooth-scrolling 24 bit probably doesn't work. */
static void et6000_fb_set_origin(struct kgi_display *dpy, uint x, uint y)
{
	IS_CRITICAL;
	static uint last_xdot = -1;
	static uint last_ydot = -1;
	static uint old_offset = -1;
	register uint xpix=0,xdot=0;
	register uint ypix=0,ydot=0;
	register uint offset;
	uint fb_width=0;

	switch (dpy->mode.request.graphtype) {
		case GT_TEXT16:
		xpix = x / dpy->mode.request.text.x;
		xdot = x % dpy->mode.request.text.x;
		ypix = y / dpy->mode.request.text.y;
		ydot = y % dpy->mode.request.text.y;
		fb_width = dpy->mode.request.virtual.x / 
			dpy->mode.request.text.x * 2;
		case GT_1BIT:
		case GT_4BIT:
		xpix = x / 8;
		xdot = x % 8;
		ypix = y;
		ydot = 0;
		fb_width = dpy->mode.request.virtual.x / 8;
		break;
		case GT_8BIT:
			xpix = x / 4;
			xdot = x % 4;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virtual.x;
		break;
		case GT_15BIT:
		case GT_16BIT:
			xpix = x / 2;
			xdot = x % 2;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virtual.x*2;
			break;
		case GT_24BIT:
			xpix = x*3 / 4;
			xdot = x % 4;    /* !! FIXME */
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virtual.x*3;
			break;
		case GT_32BIT:
			xpix = x;
			xdot = 0;
			ypix = y;
			ydot = 0;
			fb_width = dpy->mode.request.virtual.x*4;
			break;
			
		default:
		INTERNALERROR;
	}
		
 	offset = xpix + ypix * fb_width / 4;

	ENTER_CRITICAL;

	if (old_offset != offset) {

		outbCRT(offset >> 8, CRT_STARTADDR_H);
		outbCRT(offset, CRT_STARTADDR_L);
		old_offset = offset;
	}

	if (last_ydot != ydot) {
		outbCRT(ydot & CR08_ROWSCAN_MASK, CRT_PRESETROW);
		last_ydot = ydot;
	}

	if (last_xdot != xdot) {

		register uint hpan;

		if (dpy->mode.request.graphtype == GT_TEXT16 &&
		    dpy->mode.request.text.x == 9) {
			
			hpan = (xdot == 0) ? 0x08 : (xdot-1) & AR13_DATA_MASK;
		} else {
			
			hpan = (xdot & AR13_DATA_MASK) << 1;
		}
		WAIT_HVRETRACE();
		outbATC(hpan, ATC_HPAN);
		outb( ATCI_ENABLEDISPLAY, ATCI);
		last_xdot = xdot;
	}
	
	LEAVE_CRITICAL;
}

static void et6000_fb_set_split(struct kgi_display *dpy, int line)
{
	register uchar foo;
	uint8 atc;
        enable_ext();
	outbCRT(line & 0xFF, CRT_LINECOMPARE);
	foo = (line & 0x0100)
		? inbCRT(0x07) | CR07_LINECOMPARE_B8
		: inbCRT(0x07) & ~CR07_LINECOMPARE_B8;
	outbCRT(foo, 0x07);

	foo = (line & 0x0200)
		? inbCRT(0x09) | CR09_LINECOMPARE_B9
		: inbCRT(0x09) & ~CR09_LINECOMPARE_B9;
	outbCRT(foo, 0x09);

        foo = (line & 0x0400)
		? inbCRT(CRT_OVRFLW_V) | CR35_LINECOMPARE_B10
		: inbCRT(CRT_OVRFLW_V) & ~CR35_LINECOMPARE_B10;
	outbCRT(foo, CRT_OVRFLW_V);
        disable_ext();
}

#define	et6000_fb_set_frame	NULL	/* !!! implement! */

#endif /* _ET6000_FB_INC */
