/* ---------------------------------------------------------------------------
 *  Tseng Labs ET6000 text16 operators
 * --------------------------------------------------------------------------
 *
 *  Copyright (C) 1997      Jason McMullan  [jmcc@cs.cmu.edu]
 *  Copyright (C) 1997      Jim Ursetto     [ursetto@uiuc.edu]
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * ---------------------------------------------------------------------------
 *
 * ---------------------------------------------------------------------------
 *
 *  This code was derived from the following source of information:
 *  [1]   IBM VGA driver by Jason McMullan
 */

#ifndef _ET6000_TEXT16_INC_
#define _ET6000_TEXT16_INC_

#undef  DEBUG_P		/* Debug Font-pointer (_very_ verbose!) */

/* GGI includes */

/*#include <Tseng/et6000_text16.h>*/
#include "IBM/vga_text16.h"
#include "pointer_mask.h"

#define CURSOR_MASK	0x48	/* Attribute mask (bg,fg) */
#define POINTER_MASK	0xFF	/* Attribute mask (bg,fg) */

#define PCELLSIDEX	3	/* Horizontal length of pointer cell box */
#define PCELLSIDEY	3	/* Vertical length of pointer cell box */

#define PCELLTOTAL	(PCELLSIDEX * PCELLSIDEY)		
#define PCELLOF(fp, xsize, i) ((fp) + (((i) / PCELLSIDEX) * (xsize)) + ((i) % PCELLSIDEX))

typedef struct text16_point_s
{
	int	visible;	/* Should I visible the old data? */
	int	offset;		/* Offset of old point (charcells) */
	int	offset2;	/* Extra offset, due to split line */
	int	splity;         /* Split line in old point (0..PCELLSIDEY) */
	uint	maxx;		/* Max visible x of old point (charcells) */
	uint	maxy;		/* Max visible y of old point (charcells) */
	int	bitoffset;	/* Offset of old point (dots) */
	charcell data[PCELLTOTAL];	/* Data at old point */
} text16_point;
	
struct text16_data_s
{
	uint		magic;
	int		font_changed;
	unsigned long	origin;
	int		splitline;
	text16_point  	pointer;
#ifndef CURSOR_HW
	text16_point	cursor;
#endif
} text16_data[MAX_NR_DISPLAYS];

#define ORIGIN 		(data->origin)
#define SPLITLINE	(data->splitline)
#define FONTCHANGED	(data->font_changed)

#define PVISIBLE	(data->pointer.visible)
#define POFFSET1	(data->pointer.offset)
#define POFFSET2	(data->pointer.offset2)
#define PSPLITY		(data->pointer.splity)
#define PMAXX		(data->pointer.maxx)
#define PMAXY		(data->pointer.maxy)
#define PBITOFFSET	(data->pointer.bitoffset)
#define PDATA		(data->pointer.data)

#ifndef CURSOR_HW
#define CVISIBLE	(data->cursor.visible)
#define COFFSET		(data->cursor.offset)
#define CDATA		(data->cursor.data[0])
#endif

static void text16_data_init(void)
{
	memset(text16_data, 0, sizeof(struct text16_data *) * MAX_NR_DISPLAYS);
}
	
static struct text16_data_s *text16_data_realloc(struct text16_data_s *old)
{
	int i;

	if (old == NULL || old->magic != TEXT16_MAGIC) {
		for (i = 0; i < MAX_NR_DISPLAYS; i++) 
			if (text16_data[i].magic != TEXT16_MAGIC) {
				memset(&(text16_data[i]), 0,
				 sizeof(struct text16_data_s));
				text16_data[i].magic = TEXT16_MAGIC;
				return &(text16_data[i]);
			}
		return NULL;
	}
	memset(old, 0, sizeof(struct text16_data_s));
	old->magic = TEXT16_MAGIC;
	return old;
}

/* Text-font based mouse pointer code */

/* Ascii codes of font locs. to abuse for our purposes */
static unsigned char pointer_char[PCELLTOTAL] = {
	0xC5, 0xC6, 0xC7,
	0xD6, 0xD7, 0xD8,
	0xCF, 0xDD, 0xDE
};

#if DEBUG_LEVEL > 0
static void print_mask(const char *msg, uint *mask, uint texty)
{
	int i, j;
	char buff[33];

	printk("%s\n", msg);

	buff[32] = 0;
	for (i = 0; i < texty * 2; i++) {
		for (j = 0; j < 32; j++) 
			buff[31 - j] = (mask[i] & (1 << j)) ? 'O' : ' ';    
		printk("  [%.2d]%s\n", i, buff);
	}
}
#endif /* DEBUG_LEVEL > 0 */

typedef struct text16_font_reg_s 
{
	uchar SR2,SR4;
	uchar GR4,GR5,GR6;
} text16_font_reg;

#define FONT_GET 0
#define FONT_SET 1

#ifdef NO_LONGER_USED_CODE
static void text16_font_access(int getset, int chr, uchar *charcell)
{
	text16_font_reg regs;
	uchar *fontbuff = (uchar *)(phys_to_virt(0xA0000)) + (chr << 5);
	IS_CRITICAL;

#ifdef SLOW_POINTER
	WAIT_VRETRACE();
#endif
	ENTER_CRITICAL;
	regs.SR2 = inbSEQ(0x02);
	regs.SR4 = inbSEQ(0x04);	/* Save register state        */
	regs.GR4 = inbGRC(0x04);
	regs.GR5 = inbGRC(0x05);
	regs.GR6 = inbGRC(0x06);

    outbSEQ(SR02_PLANE2, 0x02);		/* Font data goes to plane 2  */
    outbSEQ(SR04_256K_ACCESS | SR04_NO_ODDEVEN, 0x04);

    outbGRC(2, 0x04);			/* Read plane 2               */
    outbGRC(GR5_WRITEMODE0, 0x05);
    outbGRC(GR6_MAP_A0_64 | GR6_GRAPHMODE, 0x06);

    if (getset == FONT_GET) 
		memcpy(charcell, fontbuff, 32);
    else
		memcpy(fontbuff, charcell, 32);

	outbSEQ(regs.SR2, 0x02);
	outbSEQ(regs.SR4, 0x04);	/* Restore registers  */
	outbGRC(regs.GR4, 0x04);
	outbGRC(regs.GR5, 0x05);
	outbGRC(regs.GR6, 0x06);
	LEAVE_CRITICAL;
}
#endif /* NO_LONGER_USED_CODE */

static inline void text16_font_enter(text16_font_reg *regs) 
{
	IS_CRITICAL;

#ifdef SLOW_POINTER
	WAIT_VRETRACE();
#endif
	ENTER_CRITICAL;
	regs->SR2 = inbSEQ(0x02);
	regs->SR4 = inbSEQ(0x04);	/* Save register state        */
	regs->GR4 = inbGRC(0x04);
	regs->GR5 = inbGRC(0x05);
	regs->GR6 = inbGRC(0x06);

	outbSEQ(SR02_PLANE2, 0x02);	/* Font data goes to plane 2  */
	outbSEQ(SR04_256K_ACCESS | SR04_NO_ODDEVEN, 0x04);

	outbGRC(2, 0x04);		/* Read plane 2               */
	outbGRC(GR5_WRITEMODE0, 0x05);
	outbGRC(GR6_MAP_A0_64 | GR6_GRAPHMODE, 0x06);
	LEAVE_CRITICAL;
}

static void inline text16_font_leave(text16_font_reg *regs)
{
	IS_CRITICAL;

	ENTER_CRITICAL;
	outbSEQ(regs->SR2, 0x02);
	outbSEQ(regs->SR4, 0x04);	/* Restore registers  */
	outbGRC(regs->GR4, 0x04);
	outbGRC(regs->GR5, 0x05);
	outbGRC(regs->GR6, 0x06);
	LEAVE_CRITICAL;
}

/* Need to change this to linear mem !!! */
static inline void direct_text16_font_read(int chr, uchar *charcell)
{
	memcpy(charcell, (uchar *)(phys_to_virt(0xA0000)) + (chr << 5), 32);
}

/* And this!! */
static inline void direct_text16_font_write(int chr, uchar *charcell)
{
	memcpy((uchar *)(phys_to_virt(0xA0000)) + (chr << 5), charcell, 32);
}

static void place_pointer(charcell *fb, uint xsize, uint dx, uint dy,
			  uint texty, int isdelt, struct text16_data_s *data)
{
	int i, j;
	uint mask[48];
    uchar font[PCELLTOTAL * 32], *cp;
	text16_font_reg regs;
	charcell *fb1 = fb + POFFSET1, *fb2 = fb + POFFSET2;
	charcell *cell = PDATA;
	uint maxx = PMAXX, maxy = PMAXY;
	int splity = PSPLITY;
    
	if (dy > 16) {
		NOTICE1("PtrHack: Can't deal with characters >16 bits high");
		return;
	}
   
#define LOOPOVERxyi { register int x,y; for (y=0;y<maxy;y++) { \
			i = y * PCELLSIDEX; \
			fb = (y < splity) ? fb1 : fb2; \
			for (x = 0; x < maxx; x++, i++) {
#define ENDLOOPxyi  }}}

    /* Has anything changed since the last time we mucked with the fonts? */ 
	if (!isdelt) {
		LOOPOVERxyi
			isdelt |= (cell[i] != *PCELLOF(fb,xsize,i));
		ENDLOOPxyi
		    
			/* Nope - same characters as we used last time.  Why redo our effort? */
			
			if (!isdelt) {
				LOOPOVERxyi
					CHARSET(PCELLOF(fb, xsize, i), pointer_char[i]);
				ENDLOOPxyi
					return;
			}
	}

    /* Get the characters cells and their associated fonts */
    LOOPOVERxyi
	cell[i] = *PCELLOF(fb, xsize, i);	
    ENDLOOPxyi

    text16_font_enter(&regs);

    LOOPOVERxyi
	direct_text16_font_read(CHAR(cell[i]), font + (i << 5));
    ENDLOOPxyi

    /* Copy the old chars, as a bitmap, into mask[]  */
    memset(mask, 0, sizeof(mask)); 
    LOOPOVERxyi
	cp = font + (i << 5);
	for (j = 0;j < texty; j++) 
		mask[j + y * texty] |= (*(cp++)) << ((3 - x) * 8);
    ENDLOOPxyi

#if DEBUG_LEVEL > 0
    print_mask("Original", mask, texty);
#endif

    /* Mask with the And and Or masks */
    for (i = 0; i < POINTER_MASK_LEN; i++) {
	mask[dy + i] |= (pointer_or_mask[i] >> dx);
	mask[dy + i] ^= (pointer_xor_mask[i] >> dx);
    }
         
#if DEBUG_LEVEL > 0
    print_mask("W/Pointer", mask, texty);
#endif

    /* Now write the new data */
    LOOPOVERxyi
	cp = font + (i << 5);
	for (j = 0; j < texty; j++) 
		*(cp++) = (mask[j + y * texty] >> ((3 - x) * 8)) & 0xFF;
    ENDLOOPxyi

    LOOPOVERxyi
	direct_text16_font_write(pointer_char[i], font + (i << 5));
    ENDLOOPxyi

    text16_font_leave(&regs);

    LOOPOVERxyi
	CHARSET(PCELLOF(fb, xsize, i), pointer_char[i]);
    ENDLOOPxyi
}

static void erase_pointer(charcell *fb, uint xsize, struct text16_data_s *data)
{
	int i;
	charcell *fb1 = fb + POFFSET1, *fb2 = fb + POFFSET2;
	uint maxx = PMAXX, maxy = PMAXY;
	int splity = PSPLITY;

	LOOPOVERxyi
		*PCELLOF(fb, xsize, i) = PDATA[i];
	ENDLOOPxyi
}

#undef LOOPOVERxyi
#undef ENDLOOPxyi

/* Font setting */

static void text16_do_set_font(struct kgi_display *dpy, int no, int from, int to, struct kgi_font *font)
{
	text16_font_reg fontregs;
	int i;
	char *fbuf, *data;
	register int cnt = 32 - font->dy;

	if (font == NULL)
		return;

	fbuf = (void *) dpy->fb.pbuf; /* (char *) phys_to_virt(0x000A0000);*/
	/*printk("font fb: %p\n", dpy->fb.pbuf);*/
	/*printk("*/
	/*if (fbuf == NULL) return;*/
	
	fbuf += (no * 0x2000) + (from << 5);
	data = font->data + font->dy * from;
	if (!data)
		return;

	text16_font_enter(&fontregs);
	/* NOTE: no printk()'s are allowed in here unless klogd is active! */

	for (i = from; i <= to; i++) {
		memmove(fbuf, data, font->dy);
		memset(fbuf + font->dy, 0, cnt);
		fbuf += 32;
		data += font->dy;
	};

	text16_font_leave(&fontregs);

#ifdef CURSOR_HW
	/* Set up cursor */
	outbCRT((inbCRT(0x0A) & ~CR0A_CURSOR_START_MASK)
		| ((font->base+1) & CR0A_CURSOR_START_MASK), 0x0A);
	outbCRT((inbCRT(0x0B) & ~CR0B_CURSOR_END_MASK)
		| ((font->base+2) & CR0B_CURSOR_END_MASK), 0x0B);
#endif
}

 static void text16_set_font(struct kgi_display *dpy, struct kgi_font *font,
			    uint from, uint to)
{
	struct text16_data_s *data = 
		(struct text16_data_s *)(dpy->driver_data);
	
        text16_do_set_font(dpy, 0, from, to, font);

	FONTCHANGED = 1;
}

static void text16_set_origin(struct kgi_display *dpy, uint x, uint y)
{
	struct text16_data_s *data = 
		(struct text16_data_s *)(dpy->driver_data);
	IS_CRITICAL;
	static uint last_xdot = -1;
	static uint last_ydot = -1;
	register uint xpix = x / dpy->mode.request.text.x;
	register uint xdot = x % dpy->mode.request.text.x;
	register uint ypix = y / dpy->mode.request.text.y;
	register uint ydot = y % dpy->mode.request.text.y;

	register uint offset = xpix + ypix*dpy->fb.size.x;

	ENTER_CRITICAL;

		if (ORIGIN != offset) {

			outbCRT(offset >> 8, CRT_STARTADDR_H);
			outbCRT(offset, CRT_STARTADDR_L);
			ORIGIN = offset;
		}

		if (last_ydot != ydot) {

			outbCRT(ydot & CR08_ROWSCAN_MASK, CRT_PRESETROW);
			last_ydot = ydot;
		}

		if (last_xdot != xdot) {

			register uint hpan;

			if (dpy->mode.request.text.x == 9) {

				hpan = (xdot == 0)
					? 0x08 : (xdot-1) & AR13_DATA_MASK;
			} else {

				hpan = xdot & AR13_DATA_MASK;
			}
			WAIT_HVRETRACE();
			outbATC(hpan, ATC_HPAN);
			outb(ATCI_ENABLEDISPLAY, ATCI);
			last_xdot = xdot;
		}

	LEAVE_CRITICAL;
}

static void text16_set_split(struct kgi_display *dpy, int line)
{
	struct text16_data_s *data = 
		(struct text16_data_s *)(dpy->driver_data);
	register uchar foo;

	outbCRT(line & 0xFF, CRT_LINECOMPARE);

	foo = (line & 0x0100)
		? inbCRT(0x07) | CR07_LINECOMPARE_B8
		: inbCRT(0x07) & ~CR07_LINECOMPARE_B8;
	outbCRT(foo, 0x07);

	foo = (line & 0x0200)
		? inbCRT(0x09) | CR09_LINECOMPARE_B9
		: inbCRT(0x09) & ~CR09_LINECOMPARE_B9;
	outbCRT(foo, 0x09);

	SPLITLINE = line;
}

#define	text16_set_frame	NULL	/* !!! implement! */

#define TEXTX           (dpy->mode.request.text.x)
#define TEXTY           (dpy->mode.request.text.y)
#define VIRTUALX        (dpy->fb.size.x)
#define VIRTUALY        (dpy->fb.size.y)
#define VISIBLEX        (dpy->mode.request.visible.x)
#define VISIBLEY        (dpy->mode.request.visible.y)

void text16_show_pointer(struct kgi_display *dpy, uint x, uint y)
{
	struct text16_data_s *data = 
		(struct text16_data_s *)(dpy->driver_data);
        ulong newoff1 = ORIGIN + (x / TEXTX) + (y / TEXTY) * VIRTUALX;
	ulong newoff2 = 0;
        register charcell *p = (void *) dpy->fb.pbuf;
	uint bitoff = x + y * VISIBLEX;
	uint dx, dy, clipx, clipy, maxx, maxy;
	int splity = PCELLSIDEY - 1;
	
	if (SPLITLINE && SPLITLINE < VIRTUALY * TEXTY &&
	    y+POINTER_MASK_LEN > SPLITLINE) {
		newoff2 = (x / TEXTX) + ((y - SPLITLINE) / TEXTY) * VIRTUALX;
		if (y >= SPLITLINE) {
			newoff1 = newoff2;
			newoff2 = 0;
		}
		else 
			splity = SPLITLINE / TEXTY - y / TEXTY + 1;
	}

	DEBUG1("PShow(%c):POFFSET1=0x%.8x POFFSET2=0x%.8x PBOFFSET=0x%.8x x=%d,y=%d\n",
		PVISIBLE ? 'V' : 'i', POFFSET1, POFFSET2, PBITOFFSET, x, y);

 	ASSERT((x < VISIBLEX) && (y < VISIBLEY));

	if (PVISIBLE) 
		erase_pointer(p, VIRTUALX, data);

	dx = x % TEXTX;
	dy = y % TEXTY;
	clipx = VIRTUALX - (x / TEXTX);
	clipy = VIRTUALY - (y / TEXTY);

	maxx = ((POINTER_MASK_WIDTH - 1 + dx) / 8) + 1;
	maxx = (maxx > clipx) ? clipx : maxx;
	maxy = ((POINTER_MASK_LEN - 1 + dy) / TEXTY) + 1;
	maxy = (maxy > clipy) ? clipy : maxy;

	if (splity < 0)
		splity = 0;
	else if (splity > maxy)
		splity = maxy;

        POFFSET1 = newoff1;
	POFFSET2 = newoff2;
	PSPLITY = splity;
	PMAXX = maxx;
	PMAXY = maxy;
	
	place_pointer(p, VIRTUALX, dx, dy, TEXTY, 
		      bitoff != PBITOFFSET || FONTCHANGED, data);

	PBITOFFSET = bitoff;
	PVISIBLE = 1;
	FONTCHANGED = 0;
}

static void text16_hide_pointer(struct kgi_display *dpy)
{
	struct text16_data_s *data = 
		(struct text16_data_s *)(dpy->driver_data);
        register charcell *p = (void *) dpy->fb.pbuf;

	DEBUG1("PHide(%c): POFFSET1=0x%.8x POFFSET2=0x%.8x PBOFFSET=0x%.8x\n",
		PVISIBLE ? 'V' : 'i', POFFSET1, POFFSET2, PBITOFFSET);

	if (PVISIBLE) {
		erase_pointer(p, VIRTUALX, data);
		PVISIBLE = 0;
	}
}

#define text16_undo_pointer     text16_hide_pointer

#ifdef CURSOR_HW
static void text16_show_cursor(struct kgi_display *dpy, uint x, uint y)
{
       	struct text16_data_s *data = 
		(struct text16_data_s *)(dpy->driver_data);
        register ulong new = ORIGIN + (x / TEXTX) + (y / TEXTY) * VIRTUALX;
 	IS_CRITICAL;

        ASSERT((x < VISIBLEX) && (y < VISIBLEY));

	ENTER_CRITICAL;
	outbCRT(new >> 8, CRT_CURSORADDR_H);
	outbCRT(new & 0xFF, CRT_CURSORADDR_L);
	LEAVE_CRITICAL;
}

static void text16_hide_cursor(struct kgi_display *dpy)
{
	IS_CRITICAL;

	ENTER_CRITICAL;
	outbCRT(0xFF, CRT_CURSORADDR_H);
	outbCRT(0xFF, CRT_CURSORADDR_L);
	LEAVE_CRITICAL;
}

/* This is badly wrong. Hardware cursor doesn't need and undo function! We 
 * should put NULL here and update kgi.c to actually check if dpy->undo_xxxx
 * are not NULL before calling them.
 */
#define text16_undo_cursor      text16_hide_cursor
#else
static void text16_show_cursor(struct kgi_display *dpy, uint x, uint y)
{
       	struct text16_data_s *data = 
		(struct text16_data_s *)(dpy->driver_data);
        register ulong newoff;
        register charcell *p = (void *) dpy->fb.pbuf;
	register charcell cell;

	ASSERT((x < VISIBLEX) && (y < VISIBLEY));

	if (SPLITLINE && y >= SPLITLINE) 
		newoff = (x / TEXTX) + ((y - SPLITLINE) / TEXTY) * VIRTUALX;
	else
		newoff = ORIGIN + (x / TEXTX) + (y / TEXTY) * VIRTUALX;

	if (!CVISIBLE || newoff != COFFSET) {
		cell = *(p + newoff);
		ATTRSET(p + newoff, ATTR(cell) ^ CURSOR_MASK);
	} else 
		return;

	if (CVISIBLE)
		*(p + COFFSET) = CDATA;
	else
		CVISIBLE = 1;

	COFFSET = newoff;
	CDATA = cell;
}

static void text16_hide_cursor(struct kgi_display *dpy)
{
	struct text16_data_s *data = 
		(struct text16_data_s *)(dpy->driver_data);
        register ushort *p = (void *) dpy->fb.pbuf;

	if (CVISIBLE)
		*(p + COFFSET) = CDATA;
	
	CVISIBLE = 0;
}

#define text16_undo_cursor      text16_hide_cursor
#endif /* CURSOR_HW */

#undef SPLITLINE
#undef ORIGIN

#undef PVISIBLE
#undef POFFSET1
#undef POFFSET2
#undef PSPLITY
#undef PMAXX
#undef PMAXY
#undef PDATA

#ifndef CURSOR_HW
#undef CVISIBLE
#undef COFFSET
#undef CDATA
#endif

#endif /* _ET6000_TEXT16_INC_ */
