/* ---------------------------------------------------------------------------
** Tseng Labs ET6000 integrated PLL driver
** ---------------------------------------------------------------------------
**
** Copyright (C) 1997, The Linux-GGI project
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** You should have received a copy of the GNU General Public License
** along with this program; see the file COPYING.  If not, write to
** the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
** 
** ---------------------------------------------------------------------------
** 
** $Log: et6000_clock.c,v $
** 
** ---------------------------------------------------------------------------
** This code was derived from the following source of information:
** [1] Trio/Virge integrated PLL driver
** [2] Prerelease VGADOC5 by Finn Thoegersen
** [3] XFree86 Tseng ET4000/6000 driver -- Copyright 1990,91 by Thomas
**     Roell, Dinkelscherben, Germany
**
*/

#include <ggi/maintainers.h>

#undef MAINTAINER
#define DRIVER_NAME     "Tseng Labs ET6000 integrated PLL clock driver"
#define DRIVER_REV      "$Revision: 0.0 $"

#define DEBUG_LEVEL     0

#include <ggi/module.h>
#include "Tseng/et6000.h"

#define CLK0_PLL 0x03

#include "pll.h"

/* xf86 recommends N2 [i.e. post-VCO or p] > 0 for stable clock */

static const struct pll_limits dclk_pll =
{
    
            14318180,                       /* Fref                         */
              { 1, 1 },                       /* Factor                       */
              /* changed this! OK so far... */
              { 100000000, 270000000 },       /* FVCO range                   */
              { 1, 3 },                       /* Post-VCO divider range       */
              { 3, 129 },                     /* Dividend range               */           
              { 3, 33 }                       /* Divisor range                */
};

static struct kgi_clock clock =
{
    
            "Tseng Labs",              /* Manufacturer         */
            "ET6000 integrated DAC",    /* Model                */
              GM_ALL,                         /* Supported modes      */
              0,                              /* Special capabilities */
              { 0x7FFF, 0x7FFF },             /* Maximum resolution   */
              CLK_TYPE_PROG,                  /* Clock type           */
              {{
		      
                      0, 135000000,    /* Clock range          */
	      }}
};

/* An SDAC command word is as follows:
 * **      [0:6]   s->mul - 2
 * **      [8:12]  s->div - 2
 * **      [13:14] s->p
 * */

#define INVALID_COMMAND ((uint) -1)

static uint pll_command(struct pll_state *s)
{
    
    ASSERT(s->mul - 2 < 128);
    ASSERT(s->div - 2 < 32);
    ASSERT(s->p < 4);
    
    return  (s->mul - 2) | ((s->div - 2) << 8) | (s->p << 13);
}

static void pll_set(struct kgi_display *dpy, uint cmd)
{
    uint8 hold;
    DEBUG1("pll_set M=%d N=%d\n", cmd&0xFF, (cmd>>8)&0xFF);
    DEBUG1("requesting clock %d\n", CLK0_PLL);
    
    /* Set DCLK PLL parameters. */
    hold = PCI_in8(dpy, PCI_CRTDACwi);
    PCI_out8(dpy, (hold & PCI67_RES) | CLK0_PLL, PCI_CRTDACwi);
    PCI_out8(dpy, cmd & 0xFF, PCI_CRTDAC);         /* Write M and N to    */
    PCI_out8(dpy, (cmd >> 8) & 0xFF, PCI_CRTDAC);  /* the clock register. */
    PCI_out8(dpy, hold, PCI_CRTDACwi);

    /* Set clock select lines */
    CLK_out8(dpy, CLK0_PLL, 0);
}

#include "pll.inc"                         
