/* ---------------------------------------------------------------------------
 * Tseng Labs ET6000 integrated ramdac driver
 * ---------------------------------------------------------------------------
 *
 * Copyright (C) 1995-1997 The Linux-GGI Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * ---------------------------------------------------------------------------
 *
 * $Log: et6000_ramdac.c,v $
 *
 * ---------------------------------------------------------------------------
 * 
 * This code was derived from the following source of information:
 * [1]  IBM VGA driver
 * 
 */

#include <ggi/maintainers.h>
/*#define	MAINTAINER	Jim_Ursetto */
#define	DRIVER_NAME	"Tseng Labs ET6000 integrated DAC driver"
#define	DRIVER_REV	"$Revision: 0.00 $"
#define	DEBUG_LEVEL	0

#include <ggi/kgi.h>
#include <ggi/module.h>
#include <ggi/ggi_commands.h>

#define	DAC_PRECISION	6

#include "IBM/vga_dac.h"
#include "IBM/clut.inc"

static struct kgi_ramdac ramdac = 
{
	"Tseng Labs",				/* Manufacturer		*/
	"ET6000",				/* Model		*/
	GM_TEXT16 | GM_4BIT | GM_8BIT	/* Supported modes	*/
	| GM_15BIT | GM_16BIT | GM_24BIT | GM_32BIT,
	FL_DAC_8BIT,		 	/* Flags		*/
	{ 0xFFFF, 0xFFFF },		/* Maximum resolution	*/
	DAC_8BIT_PB,  			/* Device type		*/
	{       0,  135000000 },	/* Normal mode		*/
	{       0,         0 },		/* 2xclk mode		*/
	{
	{	  0,  80000000 },	/* GT_TEXT16	*/
	{	  0,         0 },	/* GT_TEXT32	*/
	{         0, 135000000 },	/* GT_1BIT	*/
	{	  0, 135000000 },	/* GT_4BIT	*/
	{         0, 135000000 },	/* GT_8BIT	*/	
	{	  0,         135000000 },	/* GT_15BIT	*/
	{ 	  0,         135000000 },	/* GT_16BIT	*/
	{         0,         135000000 },	/* GT_24BIT	*/
	{	  0,         135000000 }	/* GT_32BIT	*/
	}
};

/* GGI interface */

int kgim_ramdac_init(struct kgi_card *card, struct kgi_display *dpy)
{
	NOTICE(DRIVER_NAME " rev. " DRIVER_REV);
	return EOK;
}

void kgim_ramdac_done(struct kgi_display *dpy)
{
	NOTICE(DRIVER_NAME " removed.");
}

int kgim_ramdac_check_mode (struct kgi_display *dpy, struct kgi_mode *mode,
	enum kgi_tm_cmd cmd)
{
	uint newclock = mode->dclk;
  
	switch (mode->request.graphtype) {

	case GT_TEXT16:
		break;

	case GT_1BIT:
		break;

	case GT_4BIT:
		break;   

	case GT_8BIT:
		break;
	case GT_15BIT:
	  case GT_16BIT:
	  case GT_24BIT:
	  case GT_32BIT:
	    break;
	default:
  		DEBUG2("couldn't handle graphtype %i", 
		       mode->request.graphtype);
		return -E(RAMDAC,MODE_CHECK);
	}

	switch (cmd) {

	case CMD_PROPOSE:
		mode->lclk.mul = mode->clk.mul = 1;
		mode->lclk.div = mode->clk.div = 1;
		return EOK;

	case CMD_LOWER:

		if ((newclock < ramdac.clock[mode->request.graphtype].min) ||
			(newclock < ramdac.normal.min)) {

			DEBUG2("DCLK too low (%i Hz).", newclock);
			return -E(RAMDAC,CLK_LOW_LIMIT);
		}

		if (ramdac.clock[mode->request.graphtype].max < newclock) {

			newclock = ramdac.clock[mode->request.graphtype].max;
		}
		mode->dclk = newclock;
		return EOK;

	case CMD_RAISE:
		if ((newclock > ramdac.clock[mode->request.graphtype].max) ||
			(newclock > ramdac.normal.max)) {

			DEBUG2("DCLK too high (%i Hz).", newclock);
			return -E(RAMDAC,CLK_HIGH_LIMIT);
		}

		if (newclock < ramdac.clock[mode->request.graphtype].min) {

			newclock = ramdac.clock[mode->request.graphtype].min;
		}

		if (newclock < ramdac.normal.min) {

			newclock = ramdac.normal.min;
		}

		mode->dclk = newclock;
		return EOK;

     case CMD_CHECK:
		if ((mode->lclk.mul != 1) || (mode->lclk.div != 1) ||
		    (mode->clk.mul != 1) || (mode->clk.div != 1)) {

			return -E(RAMDAC,MODE_CHECK);
		}

		if ((mode->dclk < ramdac.clock[mode->request.graphtype].min) ||
		    (mode->dclk < ramdac.normal.min)) {

			return -E(RAMDAC,CLK_LOW_LIMIT);
		}

		if ((mode->dclk > ramdac.clock[mode->request.graphtype].max) ||
		    (mode->dclk > ramdac.normal.max)) {
			
			return -E(RAMDAC,CLK_HIGH_LIMIT);
		}

		return EOK;

	default:
		INTERNAL_ERROR;
		return -E(RAMDAC,BOGUS);
	}
}

void kgim_ramdac_set_mode(struct kgi_display *dpy, struct kgi_mode *mode)
{
	ggi_clut clut;

#include <ggi/palette/mono.inc>
#include <ggi/palette/vga16.inc>
#include <ggi/palette/332.inc>

	/* !!!	This isn't really neccessary. The KGI code does set the
	** !!!	desired CLUT itself... I think. -- seeger
	*/

	switch (mode->request.graphtype) {

	case GT_1BIT:
  		DAC_out8(dpy, 0xFF, DAC_PIXEL_MASK);
		clut.data = palette_mono;
		clut.size = 2;
  		dac_set_clut(dpy, &clut, 0, 2);
		break;

	case GT_TEXT16:
	case GT_4BIT:
  		DAC_out8(dpy, 0xFF, DAC_PIXEL_MASK);
		clut.data = palette_vga16;
		clut.size = 16;
  		dac_set_clut(dpy, &clut, 0, 16);
		break;

	case GT_8BIT:
  		DAC_out8(dpy, 0xFF, DAC_PIXEL_MASK);
		clut.data = palette_332;
		clut.size = 256;
  		dac_set_clut(dpy, &clut, 0, 256);
		break;
	case GT_15BIT:        
	case GT_16BIT:
	case GT_24BIT:
    	case GT_32BIT:
	    break;
	    
	default:
		INTERNAL_ERROR;
	}
}

int kgim_ramdac_command(struct kgi_graphic *graph, uint cmd, uint arg)
{
	/* This may set 
	 * -  Hardware cursor
	 * -  CLUT (or gamma correction table)
	 */
	ggi_clut clut;

	switch (cmd) { 

	case RAMDAC_SETCLUT:
		clut.size = 256;
		clut.data = (ggi_color *)(graph->io_buf);
		dac_set_clut(graph->dev.dpy, &clut, 0, 256);
		return 0;

	case RAMDAC_GETCLUT:
		clut.size = 256;
		clut.data = (ggi_color *)(graph->io_buf);
		graph->io_res = graph->io_buf;
		dac_get_clut(graph->dev.dpy, &clut, 0, 256);
		return 0;
	}
  	return -E(RAMDAC, NOSUP_ALWAYS_CANT);  /* The ramdac can't do this */
}
